shinyServer(function(input, output,session) {          

    gotLibs <- reactive({
        a<-is.element("ggplot2", installed.packages()[,1])
        b<-is.element("grid", installed.packages()[,1])
        x <- c(a,b, a&b)
        names(x) <- c("ggplot2","grid","all")
        x
    })

    output$checkGraph <- renderText({
        if(input$whichgraph=="base") return("")
        if(gotLibs()[1]) 
            library(ggplot2)    
        else return(paste(red,"<h4>Warning: ggplot2 graphics package not found, go back to base graphics</h4>"))
        if(gotLibs()[2]) {
            library(grid)
            return("")
        }    
        else return(paste(red,"<h4>Warning: grid package not found, go back to base graphics</h4>"))

        NULL       
    }) 

   param <- reactive({
        
        if(input$ex=="Linear") {
            coef<-c(0,1,0,0)
            sigma <- 0.1
        }
        if(input$ex=="Quadratic 1") {
            coef<-c(10,1,10,0)
            sigma <- 0.1
        }    
        if(input$ex=="Quadratic 2") {
            coef<-c(10,5,-4,0)
            sigma <- 1
        } 
        if(input$ex=="Cubic") {
            coef<-c(10,3,-13,10)
            sigma <- 0.09
        }         
        if(input$ex=="Pick your own") {
            coef <- as.numeric(c(input$beta0,input$beta1,input$beta2,input$beta3))
            sigma <- input$sigma
        }
        if(input$ex=="Power 4") {
            coef <- rep(0,4)
            sigma <- 1
        }    
        c(coef,sigma)
   })
         
   f <- reactive({
        if(input$ex=="Pick your own") x <- seq(input$from,input$to,length=200)
        else x <- seq(0,1,length=200)
        coef <- param()
        y <- coef[1]+coef[2]*x+coef[3]*x^2+coef[4]*x^3
        if(input$ex=="5") 
           y <- 10*(x-0.02)*(x-0.4)*(x-0.6)*(x-0.95)+0.1*x      
        data.frame(x=x,y=y)
   })     
   
   data <- reactive({
        if(input$ex=="Pick your own") x <- seq(input$from,input$to,length=input$n)
        else x <- seq(0,1,length=input$n)
        coef <- param()
        y <- coef[1]+coef[2]*x+coef[3]*x^2+coef[4]*x^3+rnorm(input$n,0,coef[5])
        if(input$ex=="Power 4") 
           y <- 10*(x-0.02)*(x-0.4)*(x-0.6)*(x-0.95)+0.1*x+rnorm(1000,0,0.05)           
        data.frame(x=x,y=y)
   }) 
   
   calcs <- reactive({
         x <- data()[,1]  
         y <- data()[,2]
         pvals <- rep(0,4)
         fit <- lm(y~x)
         pvals[1] <- round(summary(fit)[4]$coefficients[2,4],4)          
         fit <- lm(y~poly(x,2))
         pvals[2] <- round(summary(fit)[4]$coefficients[3,4],4) 
         fit <- lm(y~poly(x,3))
         pvals[3] <- round(summary(fit)[4]$coefficients[4,4],4) 
         fit <- lm(y~poly(x,4))
         pvals[4] <- round(summary(fit)[4]$coefficients[5,4],4) 
         
         pvals 
   })
        
   output$text1 <- renderText({
         pvals <- calcs() 
         line <- "<h4>Check if constant model is enough</h5>"
         line[2] <- "<h4>fit y vs x</h4>" 
         line[3] <- paste("<h4>p-value of linear term :",pvals[1],ifelse(pvals[1]>0.05," &gt; "," &lt; "),"0.05</h4>") 
         if(pvals[1]>0.05) {
            line[4] <- "<h4>Linear term is not significant, constant model good</h5>"
            return(line)
         }
         line[4] <-"<hr>"
         line[5] <- paste("<h4>fit y vs x and x<sup>2</sup></h4>") 

         line[6] <- paste("<h4>p-value of quadratic term :",pvals[2],ifelse(pvals[2]>0.05," &gt; "," &lt; "),"0.05</h4>") 
         if(pvals[2]>0.05) {
            line[7] <- "<h4>Quadratic term is not significant, linear model good</h5>"
            return(line)
         }
         line[7] <-"<hr>"
         line[8] <- paste("<h4>fit y vs x, x<sup>2</sup> and x<sup>3</sup></h4>") 
         line[9] <- paste("<h4>p-value of cubic term :",pvals[3],ifelse(pvals[3]>0.05," &gt; "," &lt; "),"0.05</h4>") 
         if(pvals[3]>0.05) {
            line[10] <- "<h4>Cubic term is not significant, quadratic model good</h4>"
            return(line)
         }
         line[10] <-"<hr>"
         line[11] <- paste("<h4>fit y vs x, x<sup>2</sup> and x<sup>3</sup></h4>") 
         line[12] <- paste("<h4>p-value of power 4 term :",pvals[4],ifelse(pvals[4]>0.05," &gt; "," &lt; "),"0.05</h4>") 
         if(pvals[4]>0.05) {
            line[13] <- "<h4>Power 4 term is not significant, cubic model good</h4>"
            return(line)
         }
         
         line[13] <- "<h4>Model should be at least of degree 4!</h4>"
         line
            
   })           
    
   output$plot1 <- renderPlot({
          pvals <- calcs()
     if(max(pvals)<0.05) return(NULL)
     deg <- c(1:4)[pvals>0.05][1]-1
#     if(deg==0) fit <- lm(rep(mean(data()[,2]),input$n)~data()[,1])
     if(deg<=1) fit <- lm(y~x,data=data())
     if(deg>1) fit <- lm(y~poly(x,deg),data=data())

     if(input$whichgraph=="base") {
        par(mfrow=c(2,2))
        plot(f(),type="l",lwd=3,xlab="x",ylab="y",main="True Curve")
        plot(data(),pch=20,lwd=3,xlab="x",ylab="y",main="Data")
        plot(fitted(fit),residuals(fit),xlab="Fits",ylab="Residuals")
        abline(h=0,lwd=2) 
      }
      else {
         if(gotLibs()[3]) { 
              dta <- f()
              plt1 <- ggplot(dta,aes(x,y))+
                      geom_line(size=1.2)+
                      xlab("x")+ylab("f")+ggtitle("True Curve")
              dta <- data.frame(x=data()[,1],y=data()[,2])
              plt2 <- ggplot(dta,aes(x,y))+
                      geom_point()+
                      xlab("x")+ylab("y")+ggtitle("Data")
              dta <- data.frame(x=fitted(fit),y=residuals(fit))
              plt3 <- ggplot(dta,aes(x,y))+
                      geom_point()+
                      geom_abline(intercept=0,slope=0,size=1.2)+
                      xlab("Fits")+ylab("Residuals")+ggtitle("")
              pushViewport(viewport(layout = grid.layout(2, 2)))
              print(plt1, vp = viewport(layout.pos.row = 1, layout.pos.col = 1))
              print(plt2, vp = viewport(layout.pos.row = 1, layout.pos.col = 2))    
              print(plt3, vp = viewport(layout.pos.row = 2, layout.pos.col = 1))                      
         
         }
      }   
   })   
           
   output$appinfo <- renderText({ 
      HTML("<br>&nbsp;<br>&nbsp;<br>&nbsp;<br>
      <a target=\"_blank\" href=\"http://academic.uprm.edu/wrolke/myapps1.htm#polyreg\"><h4>What to do</h4></a><br>
      <h6>Written by: Dr. Wolfgang Rolke<br>Created: 12/05/2014</h6>")
      
  })
  output$explainText <- renderText({ 
        "<h5>Explanation: This demo illustrates the concept of polynomial regression<br> and overfitting</h5>"
        
  })

})

